import contextlib
import shutil
import tempfile
import unittest

import os.path as op

import macholib

DYLIB_DIRECTORY = op.join(op.dirname(__file__), "data")

FOO_DYLIB = op.join(DYLIB_DIRECTORY, "foo.dylib")
FILES_TO_INSTALL_NAME = {
    FOO_DYLIB: "foo.dylib",
    op.join(DYLIB_DIRECTORY, "foo2.dylib"): "yoyo.dylib",
}

FILES_TO_RPATHS = {
    FOO_DYLIB: [],
    op.join(DYLIB_DIRECTORY, "foo_rpath.dylib"): ["@loader_path/../lib"],
}

SIMPLE_MAIN = op.join(DYLIB_DIRECTORY, "main")
FILES_TO_DEPENDENCY_NAMES = {
    SIMPLE_MAIN: ["bar.1.0.0.dylib", "/usr/lib/libSystem.B.dylib"]
}

SIMPLE_BUNDLE = op.join(DYLIB_DIRECTORY, "foo.bundle")

NO_MACHO_FILE = op.join(DYLIB_DIRECTORY, "Makefile")

@contextlib.contextmanager
def mkdtemp():
    d = tempfile.mkdtemp()
    yield d
    shutil.rmtree(d)

class BaseMachOCommandTestCase(unittest.TestCase):
    def assert_commands_equal(self, filename, r_filename, filters=None):
        """Check that the mach-o header commands are the same in filename and
        r_filename, except for the commands which id are in filter_ids."""
        if filters is None:
            filters = {}

        r_m = macholib.MachO.MachO(r_filename)
        m = macholib.MachO.MachO(filename)

        # We don't really support multi-arch mach-o for now
        self.assertEqual(len(r_m.headers), 1)
        self.assertEqual(len(r_m.headers), len(m.headers))

        r_header = r_m.headers[0]
        header = m.headers[0]

        self.assertEqual(len(r_header.commands), len(header.commands))
        for r_cmd, cmd in zip(r_header.commands, header.commands):
            filter = filters.get(r_cmd[0].cmd, lambda x: x)
            self.assertEqual(filter(r_cmd), filter(cmd))




# Copyright (c) 2013 by Enthought, Ltd.
# All rights reserved.
import sys

import argparse

from .macho_rewriter import rewriter_factory

def list_rpaths_command(namespace):
    rewriter = rewriter_factory(namespace.macho)
    for rpath in rewriter.rpaths:
        print(rpath)

def list_install_names(namespace):
    rewriter = rewriter_factory(namespace.macho)
    print(rewriter.install_name)

def change_install_name_command(namespace):
    with rewriter_factory(namespace.macho) as rewriter:
        rewriter.install_name = namespace.install_name

def change_dependency_name_command(namespace):
    with rewriter_factory(namespace.macho) as rewriter:
        rewriter.change_dependency(namespace.old_library_pattern,
                                   namespace.new_library)

def list_dependency_name_command(namespace):
    rewriter = rewriter_factory(namespace.macho)
    for dependency in rewriter.dependencies:
        print(dependency)

def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    p = argparse.ArgumentParser()
    sub_parsers = p.add_subparsers(title="sub commands")

    rpath = sub_parsers.add_parser("list_rpaths", help="Print rpaths")
    rpath.add_argument("macho", help="Path to the mach-o to manipulate")
    rpath.set_defaults(func=list_rpaths_command)

    install_name_parser = sub_parsers.add_parser("install_name", help="Print the install name")
    install_name_parser.add_argument("macho", help="Path to the mach-o to manipulate")
    install_name_parser.set_defaults(func=list_install_names)

    change_install_name_parser = sub_parsers.add_parser("change_install_name", help="Change install_name")
    change_install_name_parser.add_argument("macho", help="Path to the mach-o to manipulate")
    change_install_name_parser.add_argument("install_name", help="New install name")
    change_install_name_parser.set_defaults(func=change_install_name_command)

    change_dependency_name_parser = sub_parsers.add_parser("change_library",
            help="Change library dependency")
    change_dependency_name_parser.add_argument("macho", help="Path to the mach-o to manipulate")
    change_dependency_name_parser.add_argument("old_library_pattern", help="Old library to change (can be a regex)")
    change_dependency_name_parser.add_argument("new_library",
            help="New library to replace the old one with") 
    change_dependency_name_parser.set_defaults(func=change_dependency_name_command)

    list_dependency_name_parser = sub_parsers.add_parser("list_libraries",
            help="Change library dependency")
    list_dependency_name_parser.add_argument("macho", help="Path to the mach-o to manipulate")
    list_dependency_name_parser.set_defaults(func=list_dependency_name_command)

    namespace = p.parse_args(argv)
    namespace.func(namespace)

if __name__ == "__main__":
    main()



import re

import macholib

from .errors import MachoError
from .dependency import _list_dependencies_macho, _change_command_data_inplace, \
        _find_lc_dylib_command
from .detect import detect_macho_type
from .misc import _install_name_macho, _change_id_dylib_command
from .rpath import _add_rpath_to_header, _list_rpaths_macho
from .utils import convert_to_string, safe_update

class _MachoRewriter(object):
    """
    Macho rewriters can be used to query and change mach-o properties relevant
    for relocatability.

    Concretely, you can query/modify the following:

        - rpaths sections
        - dependencies

    See also
    --------
    rewriter_factory which instanciates the right rewriter by auto-guessing the
    mach-o type.
    """
    def __init__(self, filename):
        self.filename = filename

        self._m = macholib.MachO.MachO(filename)
        if len(self._m.headers) == 0:
            raise MachoError("No header found ?")
        elif len(self._m.headers) > 1:
            raise MachoError("Universal binaries not yet supported")

        self._rpaths = _list_rpaths_macho(self._m)[0]
        self._dependencies = _list_dependencies_macho(self._m)[0]

    def __enter__(self):
        return self

    def __exit__(self, *a, **kw):
        self.commit()

    def commit(self):
        def writer(f):
            f.seek(0)
            self._m.headers[0].write(f)
        safe_update(self.filename, writer, "wb")

    @property
    def rpaths(self):
        """
        This is the list of defined rpaths.

        Note
        ----
        This includes the list of uncommitted changes.
        """
        return self._rpaths

    #----------
    # rpath API
    #----------
    def extend_rpaths(self, new_rpaths):
        """
        Extend the existing set of rpaths with the given list.

        Parameters
        ----------
        new_rpaths: seq
            List of rpaths (i.e. list of strings).

        Note
        ----
        The binary is not actually updated intil the sync method has been
        called.
        """
        header = self._m.headers[0]
        for rpath in new_rpaths:
            self._rpaths.append(rpath)
            _add_rpath_to_header(header, rpath)

    def append_rpath(self, new_rpath):
        """
        Append the given rpath to the existing set of rpaths.

        Parameters
        ----------
        new_rpath: str
            The new rpath.

        Note
        ----
        The binary is not actually updated intil the sync method has been
        called.
        """
        header = self._m.headers[0]
        self._rpaths.append(new_rpath)
        _add_rpath_to_header(header, new_rpath)

    def append_rpath_if_not_exists(self, new_rpath):
        """
        Append the given rpath to the existing set of rpaths, but only if it
        does not already defined in the binary.

        Parameters
        ----------
        new_rpath: str
            The new rpath.

        Note
        ----
        The binary is not actually updated until the sync method has been
        called.
        """
        header = self._m.headers[0]
        if not new_rpath in self._rpaths:
            _add_rpath_to_header(header, new_rpath)

    #-----------------
    # dependencies API
    #-----------------
    @property
    def dependencies(self):
        """
        The list of dependencies.

        Note
        ----
        This includes the list of uncommitted changes.
        """
        return self._dependencies

    def change_dependency(self, old_dependency_pattern, new_dependency,
                          ignore_error=True):
        """
        Change the dependency matching the given pattern to the new dependency
        name.

        Parameters
        ----------
        old_dependency_pattern: str
            Regex pattern to match against
        new_dependency: str
            New dependency name to replace with.
        ignore_error: bool
            If true, do not raise an exception of no dependency has been
            changed.
        """
        r_old_dependency = re.compile(old_dependency_pattern)
        old_dependencies = self._dependencies[:]

        header = self._m.headers[0]

        i_dependency = 0
        for command_index, (load_command, dylib_command, data) in \
                _find_lc_dylib_command(header, macholib.mach_o.LC_LOAD_DYLIB):

            name = convert_to_string(data)
            m = r_old_dependency.search(name)
            if m:
                _change_command_data_inplace(header, command_index,
                        (load_command, dylib_command, data), new_dependency)
                self._dependencies[i_dependency] = new_dependency

            i_dependency += 1

        if not ignore_error and old_dependencies != self._dependencies:
            raise MachoError("Pattern {0} not found in the list of dependencies".
                             format(old_dependency_pattern))

class ExecutableRewriter(_MachoRewriter):
    pass

class BundleRewriter(_MachoRewriter):
    pass

class DylibRewriter(_MachoRewriter):
    def __init__(self, filename):
        super(DylibRewriter, self).__init__(filename)

        if not self._m.headers[0].filetype == "dylib":
            raise MachoError("file {0} is not a dylib".format(filename))

        self._install_name = _install_name_macho(self._m)[0]

    @property
    def install_name(self):
        return self._install_name

    @install_name.setter
    def install_name(self, new_install_name):
        _change_id_dylib_command(self._m.headers[0], new_install_name)

        self._install_name = new_install_name

def rewriter_factory(filename):
    macho_type = detect_macho_type(filename)
    if macho_type == "dylib":
        return DylibRewriter(filename)
    elif macho_type == "execute":
        return ExecutableRewriter(filename)
    elif macho_type == "bundle":
        return BundleRewriter(filename)
    else:
        raise MachoError("file {0} is not a mach-o file !".format(filename))



# Copyright (c) 2013 by Enthought, Ltd.
# All rights reserved.
import macholib.MachO
import macholib.mach_o

from macholib.ptypes import sizeof

from .utils import convert_to_string, macho_path_as_data, safe_update

def list_rpaths(filename):
    """Get the list of rpaths defined in the given mach-o binary.

    The returned value is a list rpaths such as rpaths[i] is the list of rpath
    in the i-th header.

    Note
    ----
    The '\0' padding at the end of each rpath is stripped

    Parameters
    ----------
    filename: str
        The path to the mach-o binary file to look at
    """
    m = macholib.MachO.MachO(filename)
    return _list_rpaths_macho(m)

def _list_rpaths_macho(m):
    rpaths = []

    for header in m.headers:
        header_rpaths = []
        rpath_commands = [command for command in header.commands if
                isinstance(command[1], macholib.mach_o.rpath_command)]
        for rpath_command in rpath_commands:
            rpath = rpath_command[2]
            if not rpath.endswith(b"\x00"):
                raise ValueError("Unexpected end character for rpath command value: %r".format(rpath))
            else:
                header_rpaths.append(convert_to_string(rpath))
        rpaths.append(header_rpaths)

    return rpaths

def add_rpaths(filename, rpaths):
    """Add the given list of path rpaths to all header in a MachO file.

    Parameters
    ----------
    filename: str
        The path to the macho-o binary file to add rpath to
    rpaths: seq
        List of paths to add as rpath to the mach-o binary
    """
    macho = macholib.MachO.MachO(filename)
    for header in macho.headers:
        for rpath in rpaths:
            _add_rpath_to_header(header, rpath)

    def writer(f):
        for header in macho.headers:
            f.seek(0)
            header.write(f)
    safe_update(filename, writer, "wb")

def _add_rpath_to_header(header, rpath):
    """Add an LC_RPATH load command to a MachOHeader.

    Parameters
    ----------
    header: MachOHeader instances
        A mach-o header to add rpath to
    rpath: str
        The rpath to add to the given header
    """
    if header.header.magic in (macholib.mach_o.MH_MAGIC, macholib.mach_o.MH_CIGAM):
        pad_to = 4
    else:
        pad_to = 8
    data = macho_path_as_data(rpath, pad_to=pad_to)
    header_size = sizeof(macholib.mach_o.load_command) + sizeof(macholib.mach_o.rpath_command)

    rem = (header_size + len(data)) % pad_to
    if rem > 0:
        data += b'\x00' * (pad_to - rem)

    command_size = header_size + len(data)

    cmd = macholib.mach_o.rpath_command(header_size, _endian_=header.endian)
    lc = macholib.mach_o.load_command(macholib.mach_o.LC_RPATH, command_size,
        _endian_=header.endian)
    header.commands.append((lc, cmd, data))
    header.header.ncmds += 1
    header.changedHeaderSizeBy(command_size)



import shutil
import tempfile

import os.path as op

from macholib import mach_o

from .. import dependency

from .common import DYLIB_DIRECTORY, FILES_TO_DEPENDENCY_NAMES
from .common import BaseMachOCommandTestCase

class TestDependencies(BaseMachOCommandTestCase):
    def test_simple_read(self):
        for f, dependencies in FILES_TO_DEPENDENCY_NAMES.items():
            self.assertEqual(len(dependency.dependencies(f)), 1)
            self.assertEqual(dependency.dependencies(f)[0], dependencies)

    def test_simple_write(self):
        r_dependencies = ["bar.2.0.0.dylib", "/usr/lib/libSystem.B.dylib"]

        old_dependency = "bar.1.0.0.dylib"
        new_dependency = "bar.2.0.0.dylib"

        temp_fp = tempfile.NamedTemporaryFile()
        main = op.join(DYLIB_DIRECTORY, "main")
        with open(main, "rb") as fp:
            shutil.copyfileobj(fp, temp_fp)

        dependency.change_dependency(temp_fp.name, old_dependency, new_dependency)

        self.assertEqual(dependency.dependencies(temp_fp.name)[0], r_dependencies)
        filters = {mach_o.LC_LOAD_DYLIB: lambda x: (x[0], x[1])}
        self.assert_commands_equal(main, temp_fp.name, filters)



import unittest

from machotools.detect import is_macho, is_dylib, is_executable, is_bundle

from machotools.tests.common import FOO_DYLIB, SIMPLE_MAIN, SIMPLE_BUNDLE, NO_MACHO_FILE

class TestDetect(unittest.TestCase):
    def test_dylib(self):
        self.assertTrue(is_dylib(FOO_DYLIB))
        self.assertFalse(is_bundle(FOO_DYLIB))
        self.assertFalse(is_executable(FOO_DYLIB))

    def test_bundle(self):
        self.assertFalse(is_dylib(SIMPLE_BUNDLE))
        self.assertTrue(is_bundle(SIMPLE_BUNDLE))
        self.assertFalse(is_executable(SIMPLE_BUNDLE))

    def test_executable(self):
        self.assertFalse(is_dylib(SIMPLE_MAIN))
        self.assertFalse(is_bundle(SIMPLE_MAIN))
        self.assertTrue(is_executable(SIMPLE_MAIN))

    def test_no_macho(self):
        self.assertFalse(is_dylib(NO_MACHO_FILE))
        self.assertFalse(is_bundle(NO_MACHO_FILE))
        self.assertFalse(is_executable(NO_MACHO_FILE))

    def test_is_macho(self):
        self.assertTrue(is_macho(FOO_DYLIB))
        self.assertTrue(is_macho(SIMPLE_MAIN))
        self.assertTrue(is_macho(SIMPLE_BUNDLE))
        self.assertFalse(is_macho(NO_MACHO_FILE))



import shutil
import tempfile
import unittest

import os.path as op

from macholib import mach_o

from .. import dependency, misc
from .common import DYLIB_DIRECTORY, FILES_TO_INSTALL_NAME
from .common import BaseMachOCommandTestCase

class TestInstallName(BaseMachOCommandTestCase):
    def test_simple_read(self):
        for f, install_name in FILES_TO_INSTALL_NAME.items():
            self.assertEqual(len(misc.install_name(f)), 1)
            self.assertEqual(misc.install_name(f)[0], install_name)

    def test_simple_write(self):
        r_install_name = "youpla.dylib"

        temp_fp = tempfile.NamedTemporaryFile()
        dylib = op.join(DYLIB_DIRECTORY, "foo.dylib")
        with open(dylib, "rb") as fp:
            shutil.copyfileobj(fp, temp_fp)

        misc.change_install_name(temp_fp.name, r_install_name)

        self.assertEqual(misc.install_name(temp_fp.name)[0], r_install_name)
        filters = {mach_o.LC_ID_DYLIB: lambda x: (x[0], x[1])}
        self.assert_commands_equal(dylib, temp_fp.name, filters)

class TestDependents(unittest.TestCase):
    def test_simple(self):
        dylib = op.join(DYLIB_DIRECTORY, "foo.dylib")

        self.assertEqual(len(dependency.dependencies(dylib)), 1)
        self.assertEqual(dependency.dependencies(dylib)[0], ["/usr/lib/libSystem.B.dylib"])



# Copyright (c) 2013 by Enthought, Ltd.
# All rights reserved.
__version__ = "0.1.0.dev1"

from machotools.macho_rewriter import BundleRewriter, DylibRewriter, \
    ExecutableRewriter, rewriter_factory

# Silent pyflakes vim plugin
__all__ = [
    "BundleRewriter", "DylibRewriter", "ExecutableRewriter",
    "rewriter_factory",
    "__version__",
]

from .__version import __version__



import shutil
import tempfile
import unittest

import os.path as op

from ..rpath import add_rpaths, list_rpaths

from machotools.tests.common import DYLIB_DIRECTORY, FILES_TO_RPATHS

class TestRpath(unittest.TestCase):
    def _save_temp_copy(self, filename):
        temp_fp = tempfile.NamedTemporaryFile()
        with open(filename, "rb") as fp:
            shutil.copyfileobj(fp, temp_fp)

        return temp_fp

    def test_list_rpaths(self):
        for f, rpaths in FILES_TO_RPATHS.items():
            self.assertEqual(len(list_rpaths(f)), 1)
            self.assertEqual(list_rpaths(f)[0], rpaths)

    def test_write_rpath_empty(self):
        r_rpaths = ["@loader_path/../lib"]

        dylib = op.join(DYLIB_DIRECTORY, "foo.dylib")
        temp_fp = self._save_temp_copy(dylib)
        add_rpaths(temp_fp.name, r_rpaths)

        self.assertEqual(list_rpaths(temp_fp.name)[0], r_rpaths)

    def test_write_rpath_existing(self):
        dylib = op.join(DYLIB_DIRECTORY, "foo_rpath.dylib")
        new_rpath = "@loader_path/../lib2"

        r_rpaths = list_rpaths(dylib)[0]
        r_rpaths.append(new_rpath)

        temp_fp = tempfile.NamedTemporaryFile()
        with open(dylib, "rb") as fp:
            shutil.copyfileobj(fp, temp_fp)

        add_rpaths(temp_fp.name, [new_rpath])

        self.assertEqual(list_rpaths(temp_fp.name)[0], r_rpaths)



import re

import macholib

from macholib import mach_o

from .common import _change_command_data_inplace, _find_lc_dylib_command
from .utils import convert_to_string, safe_update

def _find_specific_lc_load_dylib(header, dependency_pattern):
    for index, (load_command, dylib_command, data) in \
            _find_lc_dylib_command(header, mach_o.LC_LOAD_DYLIB):
        m = dependency_pattern.search(convert_to_string(data))
        if m:
            return index, (load_command, dylib_command, data)

def _change_dependency_command(header, old_dependency_pattern, new_dependency):
    old_command = _find_specific_lc_load_dylib(header, old_dependency_pattern)
    if old_command is None:
        return
    command_index, command_tuple = old_command
    _change_command_data_inplace(header, command_index, command_tuple, new_dependency)

def dependencies(filename):
    """Returns the list of mach-o the given binary depends on.

    Parameters
    ----------
    filename: str
        Path to the mach-o to query

    Returns
    -------
    dependency_names: seq
        dependency_names[i] is the list of dependencies for the i-th header.
    """
    m = macholib.MachO.MachO(filename)
    return _list_dependencies_macho(m)

def _list_dependencies_macho(m):
    ret = []

    for header in m.headers:
        this_ret = []
        for load_command, dylib_command, data in header.commands:
            if load_command.cmd == mach_o.LC_LOAD_DYLIB:
                this_ret.append(convert_to_string(data))
        ret.append(this_ret)
    return ret

def change_dependency(filename, old_dependency_pattern, new_dependency):
    """Change the install name of a mach-o dylib file.

    For a multi-arch binary, every header is overwritten to the same install
    name

    Parameters
    ----------
    filename: str
        Path to the mach-o file to modify
    new_install_name: str
        New install name
    """
    _r_old_dependency = re.compile(old_dependency_pattern)
    m = macholib.MachO.MachO(filename)
    for header in m.headers:
        _change_dependency_command(header, _r_old_dependency, new_dependency)

    def writer(f):
        for header in m.headers:
            f.seek(0)
            header.write(f)
    safe_update(filename, writer, "wb")



# Copyright (c) 2013 by Enthought, Ltd.
# All rights reserved.
import sys

import macholib.MachO

from macholib import mach_o

from .utils import rstrip_null_bytes, safe_update
from .common import _change_command_data_inplace, _find_lc_dylib_command

def install_name(filename):
    """Returns the install name of a mach-o dylib file."""
    m = macholib.MachO.MachO(filename)
    return _install_name_macho(m)

def _install_name_macho(m):
    ret = []

    for header in m.headers:
        install_names = []
        for command in header.commands:
            if command[0].cmd == mach_o.LC_ID_DYLIB:
                data = rstrip_null_bytes(command[2])
                install_name = data.decode(sys.getfilesystemencoding())
                install_names.append(install_name)

        if len(install_names) != 1:
            raise ValueError("Unexpected number of LC_ID_DYLIB commands (%d)" % len(install_names))
        else:
            ret.append(install_names[0])

    return ret

def _remove_command(header, command):
    """Remove the command from the mach-o header.

    command is a 3-tuple with the same format as header.commands"""
    command_index = header.commands.index(command)
    header.commands.remove(command)
    header.header.ncmds -= 1
    header.changedHeaderSizeBy(-command[0].cmdsize)

    return command_index

def change_install_name(filename, new_install_name):
    """Change the install name of a mach-o dylib file.

    For a multi-arch binary, every header is overwritten to the same install
    name

    Parameters
    ----------
    filename: str
        Path to the mach-o file to modify
    new_install_name: str
        New install name
    """
    m = macholib.MachO.MachO(filename)
    for header in m.headers:
        _change_id_dylib_command(header, new_install_name)

    def writer(f):
        for header in m.headers:
            f.seek(0)
            header.write(f)
    safe_update(filename, writer, "wb")

def _change_id_dylib_command(header, new_install_name):
    command_index, command_tuple = _find_lc_id_dylib(header)
    _change_command_data_inplace(header, command_index, command_tuple, new_install_name)

def _find_lc_id_dylib(header):
    commands = _find_lc_dylib_command(header, mach_o.LC_ID_DYLIB)

    filename = header.parent.filename
    if len(commands) < 1:
        raise ValueError("No LC_ID_DYLIB command in {0} ?".format(filename))
    elif len(commands) > 1:
        raise ValueError("Unexpected number of LC_ID_DYLIB commands (>1) for {0}".format(filename))
    else:
        return commands[0]



from macholib.MachO import MachO

def detect_macho_type(path):
    """
    Returns None if not a mach-o.
    """
    try:
        p = MachO(path)
    except ValueError as e:
        # Grrr, why isn't macholib raising proper exceptions...
        assert str(e).startswith("Unknown Mach-O")
        return None
    else:
        if len(p.headers) < 1:
            raise ValueError("No headers in the mach-o file ?")
        else:
            return p.headers[0].filetype

def is_macho(path):
    """Return True if the given path is a Mach-O binary."""
    try:
        MachO(path)
        return True
    except ValueError as e:
        # Grrr, why isn't macholib raising proper exceptions...
        assert str(e).startswith("Unknown Mach-O")
        return False

def is_executable(path):
    """Return True if the given file is a mach-o file of type 'execute'."""
    return detect_macho_type(path) == "execute"

def is_dylib(path):
    """Return True if the given file is a mach-o file of type 'execute'."""
    return detect_macho_type(path) == "dylib"

def is_bundle(path):
    """Return True if the given file is a mach-o file of type 'bundle'."""
    return detect_macho_type(path) == "bundle"



import shutil
import unittest

import os.path as op

from .. import dependency, misc
from ..macho_rewriter import DylibRewriter, ExecutableRewriter, \
    _MachoRewriter, rewriter_factory
from ..rpath import list_rpaths
from ..tests.common import FILES_TO_DEPENDENCY_NAMES, FILES_TO_RPATHS, \
    FILES_TO_INSTALL_NAME, FOO_DYLIB, SIMPLE_MAIN, mkdtemp

class TestMachoRewriter(unittest.TestCase):
    def test_rpaths(self):
        for f, r_rpaths in FILES_TO_RPATHS.items():
            rewriter = _MachoRewriter(f)
            self.assertEqual(rewriter.rpaths, r_rpaths)

    def test_context_manager(self):
        with mkdtemp() as d:
            for f, r_rpaths in FILES_TO_RPATHS.items():
                copy = op.join(d, op.basename(f))
                shutil.copy(f, copy)

                with _MachoRewriter(copy) as rewriter:
                    rewriter.append_rpath("yomama")

                self.assertEqual(list_rpaths(copy)[0], r_rpaths + ["yomama"])

    def test_append_rpath(self):
        with mkdtemp() as d:
            for f, r_rpaths in FILES_TO_RPATHS.items():
                copy = op.join(d, op.basename(f))
                shutil.copy(f, copy)

                rewriter = _MachoRewriter(copy)
                rewriter.append_rpath("yomama")

                self.assertEqual(rewriter.rpaths, r_rpaths + ["yomama"])

                rewriter.commit()
                self.assertEqual(list_rpaths(copy)[0], r_rpaths + ["yomama"])

    def test_extend_rpaths(self):
        with mkdtemp() as d:
            for f, r_rpaths in FILES_TO_RPATHS.items():
                appended_rpaths = ["yomama", "yoyoma"]

                copy = op.join(d, op.basename(f))
                shutil.copy(f, copy)

                rewriter = _MachoRewriter(copy)
                rewriter.extend_rpaths(appended_rpaths)

                self.assertEqual(rewriter.rpaths, r_rpaths + appended_rpaths)

                rewriter.commit()
                self.assertEqual(list_rpaths(copy)[0], r_rpaths + appended_rpaths)

    def test_append_rpath_if_not_exist(self):
        with mkdtemp() as d:
            for f, r_rpaths in FILES_TO_RPATHS.items():
                copy = op.join(d, op.basename(f))
                shutil.copy(f, copy)

                rewriter = _MachoRewriter(copy)
                rewriter.append_rpath("yomama")
                rewriter.append_rpath_if_not_exists("yomama")

                self.assertEqual(rewriter.rpaths, r_rpaths + ["yomama"])

                rewriter.commit()
                self.assertEqual(list_rpaths(copy)[0], r_rpaths + ["yomama"])

                rewriter.append_rpath_if_not_exists("yemama")
                rewriter.commit()

                self.assertEqual(list_rpaths(copy)[0], r_rpaths + ["yomama", "yemama"])

    def test_dependencies(self):
        for f, dependencies in FILES_TO_DEPENDENCY_NAMES.items():
            rewriter = _MachoRewriter(f)
            self.assertEqual(rewriter.dependencies, dependencies)

    def test_change_dependency(self):
        with mkdtemp() as d:
            r_dependencies = ["bar.2.0.0.dylib", "/usr/lib/libSystem.B.dylib"]

            main = SIMPLE_MAIN
            old_dependency = "bar.1.0.0.dylib"
            new_dependency = "bar.2.0.0.dylib"

            copy = op.join(d, op.basename(main))
            shutil.copy(main, copy)

            rewriter = _MachoRewriter(copy)
            rewriter.change_dependency(old_dependency, new_dependency)

            self.assertEqual(rewriter.dependencies, r_dependencies)

            rewriter.commit()
            self.assertEqual(dependency.dependencies(copy)[0], r_dependencies)

class TestDylibRewriter(unittest.TestCase):
    def test_install_name(self):
        for f, r_install_name in FILES_TO_INSTALL_NAME.items():
            rewriter = DylibRewriter(f)
            self.assertEqual(rewriter.install_name, r_install_name)

    def test_change_install_name(self):
        with mkdtemp() as d:
            for f, r_install_name in FILES_TO_INSTALL_NAME.items():
                copy = op.join(d, op.basename(f))
                shutil.copy(f, copy)

                rewriter = DylibRewriter(copy)
                rewriter.install_name = "yomama"

                self.assertEqual(rewriter.install_name, "yomama")

                rewriter.commit()
                self.assertEqual(misc.install_name(copy)[0], "yomama")

class TestRewriterFactory(unittest.TestCase):
    def test_simple(self):
        rewriter = rewriter_factory(SIMPLE_MAIN)
        self.assertTrue(isinstance(rewriter, ExecutableRewriter))

        rewriter = rewriter_factory(FOO_DYLIB)
        self.assertTrue(isinstance(rewriter, DylibRewriter))



import os.path as op

from setuptools import setup

VERSION = "0.1.0"

if __name__ == "__main__":
    with open(op.join("machotools", "__version.py"), "wt") as fp:
        fp.write("__version__ = '{0}'".format(VERSION))

    setup(name="machotools",
          version=VERSION,
          author="David Cournapeau",
          author_email="cournape@gmail.com",
          license="BSD",
          packages=["machotools", "machotools.tests"],
          install_requires=["macholib"],
          classifiers=[
              "Programming Language :: Python :: 2.6",
              "Programming Language :: Python :: 2.7",
              "Programming Language :: Python :: 3.2",
              "Programming Language :: Python :: 3.3",
          ])



from macholib import mach_o

from .utils import macho_path_as_data

def _change_command_data_inplace(header, index, old_command, new_data):
    # We change the command 'in-place' to ensure the new dylib command is as
    # close as possible as the old one (same version, timestamp, etc...)
    (old_load_command, old_dylib_command, old_data) = old_command

    if header.header.magic in (mach_o.MH_MAGIC, mach_o.MH_CIGAM):
        pad_to = 4
    else:
        pad_to = 8
    data = macho_path_as_data(new_data, pad_to=pad_to)

    cmdsize_diff = len(data) - len(old_data)
    load_command = old_load_command
    load_command.cmdsize += cmdsize_diff

    dylib_command = old_dylib_command

    header.commands[index] = (load_command, dylib_command, data)
    header.changedHeaderSizeBy(cmdsize_diff)

def _find_lc_dylib_command(header, command_type):
    commands = []
    for command_index, (load_command, dylib_command, data) in enumerate(header.commands):
        if load_command.cmd == command_type:
            commands.append((command_index, (load_command, dylib_command, data)))

    return commands



# Copyright (c) 2013 by Enthought, Ltd.
# All rights reserved.
import os
import shutil
import stat
import sys
import uuid

from macholib.util import fsencoding

def macho_path_as_data(filename, pad_to=4):
    """ Encode a path as data for a MachO header.

    Namely, this will encode the text according to the filesystem
    encoding and zero-pad the result out to 4 bytes.

    Parameters
    ----------
    filename: str
        Path string to encode
    pad_to: int
        Number of bytes to pad the encoded string to
    """
    filename = fsencoding(filename) + b'\x00'
    rem = len(filename) % pad_to
    if rem > 0:
        filename += b'\x00' * (pad_to - rem)
    return filename

def rstrip_null_bytes(s):
    """Right-strip any null bytes at the end of the given string."""
    return s.rstrip(b'\x00')

def convert_to_string(data):
    data = rstrip_null_bytes(data)
    return data.decode(sys.getfilesystemencoding())

def safe_write(target, writer, mode="wt"):
    """a 'safe' way to write to files.

    Instead of writing directly into a file, this function writes to a
    temporary file in the same directory, and then rename the file to the
    target if no error occured.  On most platforms, rename is atomic, so this
    avoids leaving stale files in inconsistent states.

    Parameters
    ----------
    target: str
        destination to write to
    writer: callable or data
        if callable, assumed to be function which takes one argument, a file
        descriptor, and writes content to it. Otherwise, assumed to be data
        to be directly written to target.
    mode: str
        opening mode
    """
    if not callable(writer):
        data = writer
        writer = lambda fp: fp.write(data)

    file_mode = stat.S_IMODE(os.stat(target).st_mode)

    tmp_target = "%s.tmp%s" % (target, uuid.uuid4().hex)
    f = open(tmp_target, mode)
    try:
        writer(f)
    finally:
        f.close()
    os.chmod(tmp_target, file_mode)
    os.rename(tmp_target, target)

def safe_update(target, writer, mode="wt"):
    """a 'safe' way to update a file.

    Instead of writing directly into a file, this function first copies target
    to a temporary file and, writes to it, and then rename the file to the
    target if no error occured.  On most platforms, rename is atomic, so this
    avoids leaving stale files in inconsistent states.

    Parameters
    ----------
    target: str
        file to update
    writer: callable or data
        if callable, assumed to be function which takes one argument, a file
        descriptor, and writes content to it. Otherwise, assumed to be data
        to be directly written to target.
    mode: str
        opening mode
    """
    if 'b' in mode:
        target_mode = 'rb'
    else:
        target_mode = 'r'

    def writer_wrap(f):
        g = open(target, target_mode)
        try:
            shutil.copyfileobj(g, f)
        finally:
            g.close()
        return writer(f)
    return safe_write(target, writer_wrap, mode)
